
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, CHAT_FRAMES, CreateFrame, UIFrameFadeIn, UIFrameFadeOut

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat ScrollIndicator")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	for index = 1, #CHAT_FRAMES do
		self:CreateIndicator(_G["ChatFrame" .. index])
	end
end

function M:OnEnable()
	for index = 1, #CHAT_FRAMES do
		self:ConfigureChatFrame(_G["ChatFrame" .. index])
	end
end

function M:OnDisable()
	for index = 1, #CHAT_FRAMES do
		self:UpdateIndicator(_G["ChatFrame" .. index], true)
	end
end

-- Create a chat frame's scroll indicator button.
function M:CreateIndicator(ChatFrame)
	if not ChatFrame.scrollIndicator then
		ChatFrame.scrollIndicator = CreateFrame("Button", nil, ChatFrame, "SixUI_BottomButtonTemplate")
	end
end

-- Update an indicator's visibility.
function M:UpdateIndicator(ChatFrame, disabled)
	local ScrollIndicator = ChatFrame.scrollIndicator

	if disabled or ChatFrame:AtBottom() then
		ScrollIndicator:SetEnabled(false)
		ScrollIndicator:EnableMouse(false)
		UIFrameFadeOut(ScrollIndicator, 0.25, ScrollIndicator:GetAlpha(), 0)
	else
		ScrollIndicator:SetEnabled(true)
		ScrollIndicator:EnableMouse(true)
		UIFrameFadeIn(ScrollIndicator, 0.25, ScrollIndicator:GetAlpha(), 1)
	end
end

-- Configure a chat frame to update the indicator.
function M:ConfigureChatFrame(ChatFrame)
	if not self:IsHooked(ChatFrame, "PageDown") then
		self:SecureHook(ChatFrame, "PageDown", "UpdateIndicator")
	end

	if not self:IsHooked(ChatFrame, "PageUp") then
		self:SecureHook(ChatFrame, "PageUp", "UpdateIndicator")
	end

	if not self:IsHooked(ChatFrame, "ScrollDown") then
		self:SecureHook(ChatFrame, "ScrollDown", "UpdateIndicator")
	end

	if not self:IsHooked(ChatFrame, "ScrollToBottom") then
		self:SecureHook(ChatFrame, "ScrollToBottom", "UpdateIndicator")
	end

	if not self:IsHooked(ChatFrame, "ScrollToTop") then
		self:SecureHook(ChatFrame, "ScrollToTop", "UpdateIndicator")
	end

	if not self:IsHooked(ChatFrame, "ScrollUp") then
		self:SecureHook(ChatFrame, "ScrollUp", "UpdateIndicator")
	end

	self:UpdateIndicator(ChatFrame)
end
